#!/usr/bin/python

# Copyright (c) 2016, Daniel Nunes
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following
# disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
# disclaimer in the documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import str
import uguuAPI, pytest, mock


def upload_success(mock_post, mock_open, tmpdir, random=False, name=""):
    temp_file = tmpdir.join('foo.txt')
    temp_file.write("something random")  # it needs to be written to to be created.

    mock_response = mock.Mock()
    expected_response = "https://a.uguu.se/okpozq_%s" % temp_file
    mock_response.content = expected_response
    mock_response.status_code = 200

    url = 'https://uguu.se/api.php?d=upload-tool'
    file_to_upload = str(temp_file)

    mock_post.return_value = mock_response

    with open(file_to_upload) as f01:
        mock_open.return_value = mock.MagicMock(spec=f01)

    actual_response = uguuAPI.uploadfile(file_to_upload, random_name=random, name=name)

    with mock_open(file_to_upload, 'w') as f02:
        mock_post.assert_called_once_with(url, data={'randomname': str(random), 'name': name}, files={'file': f02})

    assert actual_response == expected_response


def upload_fail(mock_post, mock_open, tmpdir, random=False, name=""):
    import requests

    temp_file = tmpdir.join('foo.txt')
    temp_file.write("something random")  # it needs to be written to to be created.

    mock_response = mock.Mock()
    expected_response = ""
    mock_response.content = expected_response
    mock_response.status_code = 404
    http_error = requests.exceptions.HTTPError()
    mock_response.raise_for_status.side_effect = http_error

    url = 'https://uguu.se/api.php?d=upload-tool'
    file_to_upload = str(temp_file)

    mock_post.return_value = mock_response

    with open(file_to_upload) as f01:
        mock_open.return_value = mock.MagicMock(spec=f01)

    actual_response = uguuAPI.uploadfile(file_to_upload, random_name=random, name=name)

    with mock_open(file_to_upload, 'w') as f02:
        mock_post.assert_called_once_with(url, data={'randomname': str(random), 'name': name}, files={'file': f02})

    assert mock_response.raise_for_status.call_count == 1
    assert actual_response == expected_response


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_upload_success(mock_post, mock_open, tmpdir):
    upload_success(mock_post, mock_open, tmpdir)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_custom_upload_success(mock_post, mock_open, tmpdir):
    name = "lol.rst"
    upload_success(mock_post, mock_open, tmpdir, name=name)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_random_upload_success(mock_post, mock_open, tmpdir):
    random = True
    upload_success(mock_post, mock_open, tmpdir, random=random)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_both_upload_success(mock_post, mock_open, tmpdir):
    name = "lol.rst"
    random = True
    with pytest.warns(UserWarning):
        upload_success(mock_post, mock_open, tmpdir, name=name, random=random)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_upload_fail(mock_post, mock_open, tmpdir):
    upload_fail(mock_post, mock_open, tmpdir)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_custom_upload_fail(mock_post, mock_open, tmpdir):
    name = "lol.rst"
    upload_fail(mock_post, mock_open, tmpdir, name=name)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_random_upload_fail(mock_post, mock_open, tmpdir):
    random = True
    upload_fail(mock_post, mock_open, tmpdir, random=random)


@mock.patch('uguuAPI.open')
@mock.patch('uguuAPI.requests.post')
def test_both_upload_fail(mock_post, mock_open, tmpdir):
    name = "lol.rst"
    random = True
    with pytest.warns(UserWarning):
        upload_fail(mock_post, mock_open, tmpdir, name=name, random=random)
